@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2010-2012, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2010-2013, Sebastian Bergmann <sebastian@phpunit.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,9 +35,9 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  * @package    PHPUnit_Selenium
- * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2010-2012 Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
+ * @author     Sebastian Bergmann <sebastian@phpunit.de>
+ * @copyright  2010-2013 Sebastian Bergmann <sebastian@phpunit.de>
+ * @license    http://www.opensource.org/licenses/BSD-3-Clause  The BSD 3-Clause License
  * @link       http://www.phpunit.de/
  * @since      File available since Release 1.0.0
  */
@@ -47,10 +47,10 @@
  * the functionality required for web testing.
  *
  * @package    PHPUnit_Selenium
- * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2010-2012 Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 1.2.6
+ * @author     Sebastian Bergmann <sebastian@phpunit.de>
+ * @copyright  2010-2013 Sebastian Bergmann <sebastian@phpunit.de>
+ * @license    http://www.opensource.org/licenses/BSD-3-Clause  The BSD 3-Clause License
+ * @version    Release: 1.3.3
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 1.0.0
  *
@@ -69,6 +69,118 @@
  * @method unknown  answerOnNextPrompt()
  * @method unknown  assignId()
  * @method unknown  assignIdAndWait()
+ * @method unknown  assertAlert
+ * @method unknown  assertAlertNotPresent
+ * @method unknown  assertAlertPresent
+ * @method unknown  assertAllButtons
+ * @method unknown  assertAllFields
+ * @method unknown  assertAllLinks
+ * @method unknown  assertAllWindowIds
+ * @method unknown  assertAllWindowNames
+ * @method unknown  assertAllWindowTitles
+ * @method unknown  assertAttribute
+ * @method unknown  assertAttributeFromAllWindows
+ * @method unknown  assertBodyText
+ * @method unknown  assertChecked
+ * @method unknown  assertConfirmation
+ * @method unknown  assertConfirmationNotPresent
+ * @method unknown  assertConfirmationPresent
+ * @method unknown  assertCookie
+ * @method unknown  assertCookieByName
+ * @method unknown  assertCookieNotPresent
+ * @method unknown  assertCookiePresent
+ * @method unknown  assertCssCount
+ * @method unknown  assertCursorPosition
+ * @method unknown  assertEditable
+ * @method unknown  assertElementHeight
+ * @method unknown  assertElementIndex
+ * @method unknown  assertElementNotPresent
+ * @method unknown  assertElementPositionLeft
+ * @method unknown  assertElementPositionTop
+ * @method unknown  assertElementPresent
+ * @method unknown  assertElementWidth
+ * @method unknown  assertEval
+ * @method unknown  assertExpression
+ * @method unknown  assertHtmlSource
+ * @method unknown  assertLocation
+ * @method unknown  assertLogMessages
+ * @method unknown  assertMouseSpeed
+ * @method unknown  assertNotAlert
+ * @method unknown  assertNotAllButtons
+ * @method unknown  assertNotAllFields
+ * @method unknown  assertNotAllLinks
+ * @method unknown  assertNotAllWindowIds
+ * @method unknown  assertNotAllWindowNames
+ * @method unknown  assertNotAllWindowTitles
+ * @method unknown  assertNotAttribute
+ * @method unknown  assertNotAttributeFromAllWindows
+ * @method unknown  assertNotBodyText
+ * @method unknown  assertNotChecked
+ * @method unknown  assertNotConfirmation
+ * @method unknown  assertNotCookie
+ * @method unknown  assertNotCookieByName
+ * @method unknown  assertNotCssCount
+ * @method unknown  assertNotCursorPosition
+ * @method unknown  assertNotEditable
+ * @method unknown  assertNotElementHeight
+ * @method unknown  assertNotElementIndex
+ * @method unknown  assertNotElementPositionLeft
+ * @method unknown  assertNotElementPositionTop
+ * @method unknown  assertNotElementWidth
+ * @method unknown  assertNotEval
+ * @method unknown  assertNotExpression
+ * @method unknown  assertNotHtmlSource
+ * @method unknown  assertNotLocation
+ * @method unknown  assertNotLogMessages
+ * @method unknown  assertNotMouseSpeed
+ * @method unknown  assertNotOrdered
+ * @method unknown  assertNotPrompt
+ * @method unknown  assertNotSelectOptions
+ * @method unknown  assertNotSelectedId
+ * @method unknown  assertNotSelectedIds
+ * @method unknown  assertNotSelectedIndex
+ * @method unknown  assertNotSelectedIndexes
+ * @method unknown  assertNotSelectedLabel
+ * @method unknown  assertNotSelectedLabels
+ * @method unknown  assertNotSelectedValue
+ * @method unknown  assertNotSelectedValues
+ * @method unknown  assertNotSomethingSelected
+ * @method unknown  assertNotSpeed
+ * @method unknown  assertNotSpeedAndWait
+ * @method unknown  assertNotTable
+ * @method unknown  assertNotText
+ * @method unknown  assertNotTitle
+ * @method unknown  assertNotValue
+ * @method unknown  assertNotVisible
+ * @method unknown  assertNotWhetherThisFrameMatchFrameExpression
+ * @method unknown  assertNotWhetherThisWindowMatchWindowExpression
+ * @method unknown  assertNotXpathCount
+ * @method unknown  assertOrdered
+ * @method unknown  assertPrompt
+ * @method unknown  assertPromptNotPresent
+ * @method unknown  assertPromptPresent
+ * @method unknown  assertSelectOptions
+ * @method unknown  assertSelectedId
+ * @method unknown  assertSelectedIds
+ * @method unknown  assertSelectedIndex
+ * @method unknown  assertSelectedIndexes
+ * @method unknown  assertSelectedLabel
+ * @method unknown  assertSelectedLabels
+ * @method unknown  assertSelectedValue
+ * @method unknown  assertSelectedValues
+ * @method unknown  assertSomethingSelected
+ * @method unknown  assertSpeed
+ * @method unknown  assertSpeedAndWait
+ * @method unknown  assertTable
+ * @method unknown  assertText
+ * @method unknown  assertTextNotPresent
+ * @method unknown  assertTextPresent
+ * @method unknown  assertTitle
+ * @method unknown  assertValue
+ * @method unknown  assertVisible
+ * @method unknown  assertWhetherThisFrameMatchFrameExpression
+ * @method unknown  assertWhetherThisWindowMatchWindowExpression
+ * @method unknown  assertXpathCount
  * @method unknown  attachFile()
  * @method unknown  break()
  * @method unknown  captureEntirePageScreenshot()
@@ -263,9 +375,121 @@
  * @method unknown  uncheckAndWait()
  * @method unknown  useXpathLibrary()
  * @method unknown  useXpathLibraryAndWait()
+ * @method unknown  waitForAlert
+ * @method unknown  waitForAlertNotPresent
+ * @method unknown  waitForAlertPresent
+ * @method unknown  waitForAllButtons
+ * @method unknown  waitForAllFields
+ * @method unknown  waitForAllLinks
+ * @method unknown  waitForAllWindowIds
+ * @method unknown  waitForAllWindowNames
+ * @method unknown  waitForAllWindowTitles
+ * @method unknown  waitForAttribute
+ * @method unknown  waitForAttributeFromAllWindows
+ * @method unknown  waitForBodyText
+ * @method unknown  waitForChecked
  * @method unknown  waitForCondition()
+ * @method unknown  waitForConfirmation
+ * @method unknown  waitForConfirmationNotPresent
+ * @method unknown  waitForConfirmationPresent
+ * @method unknown  waitForCookie
+ * @method unknown  waitForCookieByName
+ * @method unknown  waitForCookieNotPresent
+ * @method unknown  waitForCookiePresent
+ * @method unknown  waitForCssCount
+ * @method unknown  waitForCursorPosition
+ * @method unknown  waitForEditable
+ * @method unknown  waitForElementHeight
+ * @method unknown  waitForElementIndex
+ * @method unknown  waitForElementNotPresent
+ * @method unknown  waitForElementPositionLeft
+ * @method unknown  waitForElementPositionTop
+ * @method unknown  waitForElementPresent
+ * @method unknown  waitForElementWidth
+ * @method unknown  waitForEval
+ * @method unknown  waitForExpression
+ * @method unknown  waitForHtmlSource
+ * @method unknown  waitForLocation
+ * @method unknown  waitForLogMessages
+ * @method unknown  waitForMouseSpeed
+ * @method unknown  waitForNotAlert
+ * @method unknown  waitForNotAllButtons
+ * @method unknown  waitForNotAllFields
+ * @method unknown  waitForNotAllLinks
+ * @method unknown  waitForNotAllWindowIds
+ * @method unknown  waitForNotAllWindowNames
+ * @method unknown  waitForNotAllWindowTitles
+ * @method unknown  waitForNotAttribute
+ * @method unknown  waitForNotAttributeFromAllWindows
+ * @method unknown  waitForNotBodyText
+ * @method unknown  waitForNotChecked
+ * @method unknown  waitForNotConfirmation
+ * @method unknown  waitForNotCookie
+ * @method unknown  waitForNotCookieByName
+ * @method unknown  waitForNotCssCount
+ * @method unknown  waitForNotCursorPosition
+ * @method unknown  waitForNotEditable
+ * @method unknown  waitForNotElementHeight
+ * @method unknown  waitForNotElementIndex
+ * @method unknown  waitForNotElementPositionLeft
+ * @method unknown  waitForNotElementPositionTop
+ * @method unknown  waitForNotElementWidth
+ * @method unknown  waitForNotEval
+ * @method unknown  waitForNotExpression
+ * @method unknown  waitForNotHtmlSource
+ * @method unknown  waitForNotLocation
+ * @method unknown  waitForNotLogMessages
+ * @method unknown  waitForNotMouseSpeed
+ * @method unknown  waitForNotOrdered
+ * @method unknown  waitForNotPrompt
+ * @method unknown  waitForNotSelectOptions
+ * @method unknown  waitForNotSelectedId
+ * @method unknown  waitForNotSelectedIds
+ * @method unknown  waitForNotSelectedIndex
+ * @method unknown  waitForNotSelectedIndexes
+ * @method unknown  waitForNotSelectedLabel
+ * @method unknown  waitForNotSelectedLabels
+ * @method unknown  waitForNotSelectedValue
+ * @method unknown  waitForNotSelectedValues
+ * @method unknown  waitForNotSomethingSelected
+ * @method unknown  waitForNotSpeed
+ * @method unknown  waitForNotSpeedAndWait
+ * @method unknown  waitForNotTable
+ * @method unknown  waitForNotText
+ * @method unknown  waitForNotTitle
+ * @method unknown  waitForNotValue
+ * @method unknown  waitForNotVisible
+ * @method unknown  waitForNotWhetherThisFrameMatchFrameExpression
+ * @method unknown  waitForNotWhetherThisWindowMatchWindowExpression
+ * @method unknown  waitForNotXpathCount
+ * @method unknown  waitForOrdered
  * @method unknown  waitForPageToLoad()
  * @method unknown  waitForPopUp()
+ * @method unknown  waitForPrompt
+ * @method unknown  waitForPromptNotPresent
+ * @method unknown  waitForPromptPresent
+ * @method unknown  waitForSelectOptions
+ * @method unknown  waitForSelectedId
+ * @method unknown  waitForSelectedIds
+ * @method unknown  waitForSelectedIndex
+ * @method unknown  waitForSelectedIndexes
+ * @method unknown  waitForSelectedLabel
+ * @method unknown  waitForSelectedLabels
+ * @method unknown  waitForSelectedValue
+ * @method unknown  waitForSelectedValues
+ * @method unknown  waitForSomethingSelected
+ * @method unknown  waitForSpeed
+ * @method unknown  waitForSpeedAndWait
+ * @method unknown  waitForTable
+ * @method unknown  waitForText
+ * @method unknown  waitForTextNotPresent
+ * @method unknown  waitForTextPresent
+ * @method unknown  waitForTitle
+ * @method unknown  waitForValue
+ * @method unknown  waitForVisible
+ * @method unknown  waitForWhetherThisFrameMatchFrameExpression
+ * @method unknown  waitForWhetherThisWindowMatchWindowExpression
+ * @method unknown  waitForXpathCount
  * @method unknown  windowFocus()
  * @method unknown  windowMaximize()
  */
@@ -364,11 +588,11 @@ abstract class PHPUnit_Extensions_SeleniumTestCase extends PHPUnit_Framework_Tes
      * @param  array  $browser
      * @throws InvalidArgumentException
      */
-    public function __construct($name = NULL, array $data = array(), $dataName = '')
+    public function __construct($name = NULL, array $data = array(), $dataName = '', array $browser = array())
     {
         parent::__construct($name, $data, $dataName);
         $this->testId = md5(uniqid(rand(), TRUE));
-        $this->getDriver(array());
+        $this->getDriver($browser);
     }
 
     public function setupSpecificBrowser(array $browser)
@@ -557,6 +781,7 @@ abstract class PHPUnit_Extensions_SeleniumTestCase extends PHPUnit_Framework_Tes
             $result = parent::runTest();
         } else {
             $this->runSelenese($this->getName(FALSE));
+            $result = NULL;
         }
 
         if (!empty($this->verificationErrors)) {
@@ -852,88 +1077,11 @@ abstract class PHPUnit_Extensions_SeleniumTestCase extends PHPUnit_Framework_Tes
      */
     protected function getCodeCoverage()
     {
-        if (!empty($this->coverageScriptUrl)) {
-            $url = sprintf(
-              '%s?PHPUNIT_SELENIUM_TEST_ID=%s',
-              $this->coverageScriptUrl,
-              $this->testId
-            );
-
-            $buffer = @file_get_contents($url);
-
-            if ($buffer !== FALSE) {
-                $coverageData = unserialize($buffer);
-                if (is_array($coverageData)) {
-                    return $this->matchLocalAndRemotePaths($coverageData);
-                } else {
-                    throw new Exception('Empty or invalid code coverage data received from url "' . $url . '"');
-                }
-            }
-        }
-
-        return array();
-    }
-
-    /**
-     * @param  array $coverage
-     * @return array
-     * @author Mattis Stordalen Flister <mattis@xait.no>
-     */
-    protected function matchLocalAndRemotePaths(array $coverage)
-    {
-        $coverageWithLocalPaths = array();
-
-        foreach ($coverage as $originalRemotePath => $data) {
-            $remotePath = $originalRemotePath;
-            $separator  = $this->findDirectorySeparator($remotePath);
-
-            while (!($localpath = PHPUnit_Util_Filesystem::fileExistsInIncludePath($remotePath)) &&
-                   strpos($remotePath, $separator) !== FALSE) {
-                $remotePath = substr($remotePath, strpos($remotePath, $separator) + 1);
-            }
-
-            if ($localpath && md5_file($localpath) == $data['md5']) {
-                $coverageWithLocalPaths[$localpath] = $data['coverage'];
-            }
-        }
-
-        return $coverageWithLocalPaths;
-    }
-
-    /**
-     * @param  string $path
-     * @return string
-     * @author Mattis Stordalen Flister <mattis@xait.no>
-     */
-    protected function findDirectorySeparator($path)
-    {
-        if (strpos($path, '/') !== FALSE) {
-            return '/';
-        }
-
-        return '\\';
-    }
-
-    /**
-     * @param  string $path
-     * @return array
-     * @author Mattis Stordalen Flister <mattis@xait.no>
-     */
-    protected function explodeDirectories($path)
-    {
-        return explode($this->findDirectorySeparator($path), dirname($path));
-    }
-
-    /**
-     * @param  string $directory
-     * @param  string $suffix
-     * @return array
-     */
-    protected static function getSeleneseFiles($directory, $suffix)
-    {
-        $facade = new File_Iterator_Facade;
-
-        return $facade->getFilesAsArray($directory, $suffix);
+        $coverage = new PHPUnit_Extensions_SeleniumCommon_RemoteCoverage(
+            $this->coverageScriptUrl,
+            $this->testId
+        );
+        return $coverage->get();
     }
 
     /**
@@ -961,11 +1109,12 @@ abstract class PHPUnit_Extensions_SeleniumTestCase extends PHPUnit_Framework_Tes
 
         try {
             $this->restoreSessionStateAfterFailedTest();
-
-            $buffer  = 'Current URL: ' . $this->drivers[0]->getLocation() .
-                       "\n";
+            $buffer = '';
 
             if ($this->captureScreenshotOnFailure) {
+                $buffer .= 'Current URL: ' . $this->drivers[0]->getLocation() .
+                       "\n";
+
                 $screenshotInfo = $this->takeScreenshot();
                 if ($screenshotInfo != '') {
                     $buffer .= $screenshotInfo;
@@ -988,7 +1137,7 @@ abstract class PHPUnit_Extensions_SeleniumTestCase extends PHPUnit_Framework_Tes
 
         // gain the screenshot path, lose the stack trace
         if ($this->captureScreenshotOnFailure) {
-            throw new PHPUnit_Framework_Error($buffer, $e->getCode(), $e->getFile(), $e->getLine(), $e->getTrace());
+            throw new PHPUnit_Framework_Error($buffer, $e->getCode(), $e->getFile(), $e->getLine(), $e);
         }
 
         // yes to stack trace and everything
@@ -1000,7 +1149,7 @@ abstract class PHPUnit_Extensions_SeleniumTestCase extends PHPUnit_Framework_Tes
 
         // yes to stack trace, only for F tests
         // PHPUnit issue 471 prevents getTrace() from being useful
-        throw new PHPUnit_Framework_Error($buffer, $e->getCode(), $e->getFile(), $e->getLine(), $e->getTrace());
+        throw new PHPUnit_Framework_Error($buffer, $e->getCode(), $e->getFile(), $e->getLine(), $e);
     }
 
     private function restoreSessionStateAfterFailedTest()
@@ -1029,7 +1178,7 @@ abstract class PHPUnit_Extensions_SeleniumTestCase extends PHPUnit_Framework_Tes
      * Return an empty string if the screenshotPath and screenshotUrl
      * properties are empty.
      * Issue #88.
-     * 
+     *
      * @access protected
      * @return string
      */
@@ -1047,4 +1196,13 @@ abstract class PHPUnit_Extensions_SeleniumTestCase extends PHPUnit_Framework_Tes
             return '';
         }
     }
+
+    /**
+     * Pause support for runSelenese() HTML cases
+     * @param $milliseconds
+     */
+    protected function pause($milliseconds)
+    {
+		    sleep(round($milliseconds/1000));
+	  }
 }
