@@ -2,7 +2,7 @@
 /**
  * PHPUnit
  *
- * Copyright (c) 2010-2012, Sebastian Bergmann <sb@sebastian-bergmann.de>.
+ * Copyright (c) 2010-2013, Sebastian Bergmann <sebastian@phpunit.de>.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -35,9 +35,9 @@
  * POSSIBILITY OF SUCH DAMAGE.
  *
  * @package    PHPUnit_Selenium
- * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2010-2012 Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
+ * @author     Sebastian Bergmann <sebastian@phpunit.de>
+ * @copyright  2010-2013 Sebastian Bergmann <sebastian@phpunit.de>
+ * @license    http://www.opensource.org/licenses/BSD-3-Clause  The BSD 3-Clause License
  * @link       http://www.phpunit.de/
  * @since      File available since Release 1.0.0
  */
@@ -46,10 +46,10 @@
  * Implementation of the Selenium RC client/server protocol.
  *
  * @package    PHPUnit_Selenium
- * @author     Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @copyright  2010-2012 Sebastian Bergmann <sb@sebastian-bergmann.de>
- * @license    http://www.opensource.org/licenses/bsd-license.php  BSD License
- * @version    Release: 1.2.6
+ * @author     Sebastian Bergmann <sebastian@phpunit.de>
+ * @copyright  2010-2013 Sebastian Bergmann <sebastian@phpunit.de>
+ * @license    http://www.opensource.org/licenses/BSD-3-Clause  The BSD 3-Clause License
+ * @version    Release: 1.3.3
  * @link       http://www.phpunit.de/
  * @since      Class available since Release 1.0.0
  */
@@ -145,6 +145,11 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
      */
     protected $verificationErrors = array();
 
+    /**
+     * @var array
+     */
+    private $webDriverCapabilities;
+
     public function __construct()
     {
         if (empty(self::$autoGeneratedCommands)) {
@@ -152,6 +157,14 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
         }
     }
 
+    /**
+     * Only browserName is supported.
+     */
+    public function setWebDriverCapabilities(array $capabilities)
+    {
+        $this->webDriverCapabilities = $capabilities;
+    }
+
     /**
      * @return string
      */
@@ -163,6 +176,20 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
             );
         }
 
+        if ($this->webDriverCapabilities !== NULL) {
+            $seleniumServerUrl = PHPUnit_Extensions_Selenium2TestCase_URL::fromHostAndPort($this->host, $this->port);
+            $driver = new PHPUnit_Extensions_Selenium2TestCase_Driver($seleniumServerUrl);
+            $session = $driver->startSession($this->webDriverCapabilities, new PHPUnit_Extensions_Selenium2TestCase_URL($this->browserUrl));
+            $webDriverSessionId = $session->id();
+            $this->sessionId = $this->getString(
+              'getNewBrowserSession',
+              array($this->browser, $this->browserUrl, '',
+                "webdriver.remote.sessionid=$webDriverSessionId")
+            );
+
+            $this->doCommand('setTimeout', array($this->seleniumTimeout * 1000));
+        }
+
         if (!isset($this->sessionId)) {
             $this->sessionId = $this->getString(
               'getNewBrowserSession',
@@ -248,6 +275,14 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
         $this->name = $name;
     }
 
+    /**
+     * @return string
+     */
+    public function getName()
+    {
+        return $this->name;
+    }
+
     /**
      * @param  string $browser
      * @throws InvalidArgumentException
@@ -261,6 +296,14 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
         $this->browser = $browser;
     }
 
+		/**
+		 * @return string
+		 */
+		public function getBrowser()
+		{
+			return $this->browser;
+		}
+
     /**
      * @param  string $browserUrl
      * @throws InvalidArgumentException
@@ -624,6 +667,8 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
      * @method unknown  useXpathLibrary()
      * @method unknown  useXpathLibraryAndWait()
      * @method unknown  waitForCondition()
+     * @method unknown  waitForElementPresent()
+     * @method unknown  waitForElementNotPresent()
      * @method unknown  waitForPageToLoad()
      * @method unknown  waitForPopUp()
      * @method unknown  windowFocus()
@@ -877,6 +922,8 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
             break;
 
             case 'waitForCondition':
+            case 'waitForElementPresent':
+            case 'waitForElementNotPresent':
             case 'waitForFrameToLoad':
             case 'waitForPopUp': {
                 if (count($arguments) == 1) {
@@ -914,10 +961,11 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
      *
      * @param  string $command
      * @param  array  $arguments
+     * @param  array  $namedArguments
      * @return string
      * @author Seth Casana <totallymeat@gmail.org>
      */
-    protected function doCommand($command, array $arguments = array())
+    protected function doCommand($command, array $arguments = array(), array $namedArguments = array())
     {
         $url = sprintf(
           'http://%s:%s/selenium-server/driver/',
@@ -936,6 +984,9 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
                 $postData .= sprintf('&%s=%s', $argNum, urlencode(trim($arguments[$i])));
             }
         }
+        foreach ($namedArguments as $key => $value) {
+            $postData .= sprintf('&%s=%s', $key, urlencode($value));
+        }
 
         if (isset($this->sessionId)) {
             $postData .= sprintf('&%s=%s', 'sessionId', $this->sessionId);
@@ -1118,16 +1169,16 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
         $method = $info['originalMethod'];
         $requiresTarget = $info['requiresTarget'];
         $result = $this->__call($method, $arguments);
+        $message = "Failed command: " . $command . "('"
+            . (array_key_exists(0, $arguments) ? $arguments[0] . "'" : '')
+            . (array_key_exists(1, $arguments) ? ", '" . $arguments[1] . "'" : '')
+            . ")";
 
         if ($info['isBoolean']) {
             if (!isset($info['negative']) || !$info['negative']) {
-                PHPUnit_Framework_Assert::assertTrue(
-                  $result, $arguments[count($arguments) - 1]
-                );
+                PHPUnit_Framework_Assert::assertTrue($result, $message);
             } else {
-                PHPUnit_Framework_Assert::assertFalse(
-                  $result, $arguments[count($arguments) - 1]
-                );
+                PHPUnit_Framework_Assert::assertFalse($result, $message);
             }
         } else {
             if ($requiresTarget === TRUE) {
@@ -1140,9 +1191,9 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
                 $expected = substr($expected, strlen('exact:'));
 
                 if (!isset($info['negative']) || !$info['negative']) {
-                    PHPUnit_Framework_Assert::assertEquals($expected, $result);
+                    PHPUnit_Framework_Assert::assertEquals($expected, $result, $message);
                 } else {
-                    PHPUnit_Framework_Assert::assertNotEquals($expected, $result);
+                    PHPUnit_Framework_Assert::assertNotEquals($expected, $result, $message);
                 }
             } else {
                 $caseInsensitive = FALSE;
@@ -1161,9 +1212,9 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
                         $expected = substr($expected, strlen('glob:'));
                     }
 
-                    $expected = str_replace(
+                    $expected = '^' . str_replace(
                       array('*', '?'), array('.*', '.?'), $expected
-                    );
+                    ) . '$';
                 }
 
                 $expected = '/' . str_replace('/', '\/', $expected) . '/';
@@ -1174,11 +1225,11 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
 
                 if (!isset($info['negative']) || !$info['negative']) {
                     PHPUnit_Framework_Assert::assertRegExp(
-                      $expected, $result
+                      $expected, $result, $message
                     );
                 } else {
                     PHPUnit_Framework_Assert::assertNotRegExp(
-                      $expected, $result
+                      $expected, $result, $message
                     );
                 }
             }
@@ -1198,9 +1249,13 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
 
     protected function waitForCommand($command, $arguments, $info)
     {
+        $lastExceptionMessage = '';
         for ($second = 0; ; $second++) {
             if ($second > $this->httpTimeout) {
-                PHPUnit_Framework_Assert::fail('timeout');
+                PHPUnit_Framework_Assert::fail(
+                    "WaitFor timeout. \n"
+                    . "Last exception message: \n" . $lastExceptionMessage
+                );
             }
 
             try {
@@ -1209,6 +1264,7 @@ class PHPUnit_Extensions_SeleniumTestCase_Driver
             }
 
             catch (Exception $e) {
+                $lastExceptionMessage = $e->getMessage();
             }
 
             sleep(1);
